"use strict";
/* eslint-disable no-new-func */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getComponentSchema = exports.getComponentsInfo = void 0;
const common_1 = require("../../weapps-core/utils/common");
const cals_1 = require("@cloudbase/cals");
const lodash_1 = require("lodash");
const config_1 = require("../config");
const util_1 = require("../util");
function getComponentsInfo(dependencies) {
    const outputObj = {};
    dependencies.forEach(({ name: materialName, components, isComposite }) => {
        if (isComposite) {
            components.forEach((component) => {
                let { lowCodes, componentInstances, ...compItem } = component;
                const sourceKey = `${materialName}:${compItem.name}`;
                outputObj[sourceKey] = {
                    isComposite,
                    ...compItem,
                };
            });
        }
        else {
            components.forEach(({ name, meta, emitEvents, events = undefined, isContainer, compConfig, dataForm }) => {
                outputObj[`${materialName}:${name}`] = {
                    isComposite,
                    ...meta,
                    events: events,
                    emitEvents: emitEvents,
                    isContainer,
                    compConfig,
                    dataForm,
                };
            });
        }
    });
    return outputObj;
}
exports.getComponentsInfo = getComponentsInfo;
function isSlot(comp) {
    return !comp['module'] && !comp.component;
}
function getInputProps(componentsMetaMap) {
    const outputObj = {};
    for (let key in componentsMetaMap) {
        let component = componentsMetaMap[key];
        const sourceKey = key;
        if (component.isComposite) {
            let compItem = component;
            Object.keys(compItem.dataForm || {}).forEach((key) => {
                var _a, _b;
                const inputProps = ((_a = compItem.dataForm[key]) === null || _a === void 0 ? void 0 : _a.inputProp) || ((_b = compItem.dataForm[key]) === null || _b === void 0 ? void 0 : _b.syncProp);
                if (inputProps) {
                    outputObj[sourceKey] = {
                        [key]: inputProps,
                        ...(outputObj[sourceKey] || {}),
                    };
                }
            });
        }
        else {
            const inputProps = (component === null || component === void 0 ? void 0 : component.inputProps) || (component === null || component === void 0 ? void 0 : component.syncProps);
            if (inputProps) {
                outputObj[sourceKey] = inputProps;
            }
        }
    }
    return outputObj;
}
/**
 * utils end
 */
function splitDynamicMap(data) {
    const staticData = {};
    const dataBinds = [];
    for (let key in data) {
        let meta = data[key];
        if (!meta.type || meta.type === cals_1.PropBindType.static) {
            staticData[key] = meta.value;
        }
        else if (meta.type === cals_1.PropBindType.slot) {
            continue;
        }
        else {
            dataBinds.push({
                type: meta.type,
                bindDataPath: meta.value,
                propertyPath: key,
            });
        }
    }
    return { staticData, dataBinds };
}
function walkThroughCals(cals, process) {
    var _a;
    let { items = [] } = cals;
    cals.extra = cals.extra || {};
    if (((_a = cals === null || cals === void 0 ? void 0 : cals.directives) === null || _a === void 0 ? void 0 : _a[':if']) === false) {
        cals.items = [];
    }
    for (let item of items) {
        item.extra = item.extra || {};
        item.extra.__parent = cals;
        walkThroughCals(item, process);
        item.extra.__parent = null;
    }
    process(cals);
    const { resources, directives, ...restCasl } = cals;
    delete cals.resources;
    delete cals.directives;
    return restCasl;
}
function getComponentSchema(ctx, page, componentsInfoMap = {}, wrapperClass, options) {
    const { processCssUnit = 'rem', rpxCalcRoot = `var(${config_1.CALC_RPX_ROOT_VAL_NAME})` } = options || {};
    const componentInputProps = getInputProps(componentsInfoMap) || {};
    const copyJson = JSON.parse(JSON.stringify(page));
    const compWidgets = {};
    const compDataBinds = {};
    const isComposite = (options === null || options === void 0 ? void 0 : options.isComposite) || false;
    const componentSchemaJson = walkThroughCals(copyJson, (component) => {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o;
        if (isSlot(component)) {
            return;
        }
        /**
         * 进行数据补全
         * 实际上应该在render中集中进行补全或兼容
         */
        if ((component === null || component === void 0 ? void 0 : component.component) && !(component === null || component === void 0 ? void 0 : component.module)) {
            component.module = cals_1.OFFICIAL_LIB_KEY;
        }
        const { component: componentName, module = cals_1.OFFICIAL_LIB_KEY, attributes = {} } = component;
        const { commonStyle = {} } = component.extra || {};
        const processedCommonStyle = attributes.style
            ? attributes.style
            : (0, cals_1.processCommonStyle2CSSProperties)(commonStyle, {
                defaultUnit: processCssUnit,
                rpxCalcRoot,
            });
        if (processedCommonStyle) {
            component.extra.commonStyle = processedCommonStyle;
        }
        const sourceKey = `${module}:${componentName}`;
        // 如果是复合组件的根节点，则补充 wrapperClass
        if (isComposite) {
            if (((_a = component === null || component === void 0 ? void 0 : component.extra) === null || _a === void 0 ? void 0 : _a.__parent) && !((_d = (_c = (_b = component === null || component === void 0 ? void 0 : component.extra) === null || _b === void 0 ? void 0 : _b.__parent) === null || _c === void 0 ? void 0 : _c.extra) === null || _d === void 0 ? void 0 : _d.__parent)) {
                if (!component.attributes) {
                    component.attributes = {};
                }
                component.attributes.class = Array.from(new Set([component.attributes.class, wrapperClass === null || wrapperClass === void 0 ? void 0 : wrapperClass.split(/\s/)]))
                    .filter((item) => !!item)
                    .join(' ');
            }
        }
        const className = ((_e = component === null || component === void 0 ? void 0 : component.attributes) === null || _e === void 0 ? void 0 : _e.class) || '';
        const deserializeCtx = { page };
        let dynamicStyle = component[':style']
            ? (0, cals_1.deserializeValue)(deserializeCtx, ':style', component[':style']).value
            : undefined;
        let dynamicClassList = component[':class']
            ? (0, cals_1.deserializeValue)(deserializeCtx, ':class', component[':class']).value
            : undefined;
        let dynamicIf = ((_f = component.directives) === null || _f === void 0 ? void 0 : _f[':if'])
            ? (0, cals_1.deserializeValue)(deserializeCtx, ':if', (_g = component.directives) === null || _g === void 0 ? void 0 : _g[':if']).value
            : undefined;
        let dynamicFor = ((_h = component.directives) === null || _h === void 0 ? void 0 : _h[':for'])
            ? (0, cals_1.deserializeValue)(deserializeCtx, ':for', (_j = component.directives) === null || _j === void 0 ? void 0 : _j[':for']).value
            : undefined;
        if (dynamicIf != undefined &&
            (dynamicIf.value == undefined || dynamicIf.value === true || dynamicIf.value === false)) {
            dynamicIf = undefined;
        }
        if (dynamicFor) {
            let { type, value } = dynamicFor;
            if (type === cals_1.PropBindType.expression) {
                dynamicFor.value = value.replace(/^(.*?in\s)/, '');
            }
        }
        const data = (0, cals_1.deserializeAttributes)(deserializeCtx, attributes);
        const { dataBinds, staticData } = splitDynamicMap(data);
        const componentInfo = componentsInfoMap[sourceKey];
        if (componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.compConfig) {
            component.extra.compConfig = componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.compConfig;
        }
        if (componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.events) {
            component.extra.emitEvents = componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.events.map((item) => item.name);
        }
        else if (componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.emitEvents) {
            component.extra.emitEvents = componentInfo.emitEvents.map((item) => item.eventName);
        }
        // 生成 widgets/dataBinds
        if (!isSlot(component) && component.id) {
            const parentComponent = component.extra.__parent;
            let _visible = undefined;
            switch ((_k = component.directives) === null || _k === void 0 ? void 0 : _k[':if']) {
                case true:
                case false: {
                    _visible = (_l = component.directives) === null || _l === void 0 ? void 0 : _l[':if'];
                    break;
                }
                case undefined: {
                    _visible = true;
                    break;
                }
            }
            compWidgets[component.id] = {
                ...Object.entries(staticData).reduce(function (data, [propsKey, propsValue]) {
                    let value = propsValue;
                    if (value && Array.isArray(value)) {
                        value = value.map((_, index) => {
                            var _a, _b;
                            const block = (_a = attributes === null || attributes === void 0 ? void 0 : attributes[propsKey]) === null || _a === void 0 ? void 0 : _a[index];
                            const blockInstances = (_b = block === null || block === void 0 ? void 0 : block.selectableBlock) === null || _b === void 0 ? void 0 : _b.listeners;
                            if (blockInstances) {
                                block.selectableBlock['listenerInstances'] = generateListnerInstances(ctx, blockInstances || [], {
                                    isComposite,
                                    looseError: options === null || options === void 0 ? void 0 : options.looseError,
                                });
                            }
                            return block;
                        });
                    }
                    data[propsKey] = value;
                    return data;
                }, {}),
                _visible,
                style: { ...processedCommonStyle },
                classList: className.split(/\s/).filter((item) => !!item),
                widgetType: sourceKey,
                _parentId: parentComponent
                    ? isSlot(parentComponent)
                        ? (_o = (_m = parentComponent.extra) === null || _m === void 0 ? void 0 : _m.__parent) === null || _o === void 0 ? void 0 : _o.id
                        : parentComponent.id
                    : null,
                getConfig: () => {
                    var _a;
                    return (_a = componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.compConfig) !== null && _a !== void 0 ? _a : {};
                },
            };
            const processedDataBinds = [...dataBinds];
            if (dynamicIf === null || dynamicIf === void 0 ? void 0 : dynamicIf.value) {
                processedDataBinds.push({
                    type: dynamicIf.type,
                    bindDataPath: dynamicIf.value,
                    propertyPath: '_visible',
                });
            }
            if (dynamicFor === null || dynamicFor === void 0 ? void 0 : dynamicFor.value) {
                processedDataBinds.push({
                    type: dynamicFor.type,
                    bindDataPath: dynamicFor.value,
                    propertyPath: '_waFor',
                });
            }
            if (dynamicStyle === null || dynamicStyle === void 0 ? void 0 : dynamicStyle.value) {
                processedDataBinds.push({
                    type: dynamicStyle.type,
                    bindDataPath: dynamicStyle.value,
                    propertyPath: 'style',
                });
            }
            if (dynamicClassList) {
                processedDataBinds.push({
                    type: dynamicClassList.type,
                    bindDataPath: dynamicClassList.value,
                    propertyPath: 'classList',
                });
            }
            if (processedDataBinds.length > 0) {
                compDataBinds[component.id] = generateDataBinds(ctx, processedDataBinds, {
                    isComposite,
                    looseError: options === null || options === void 0 ? void 0 : options.looseError,
                });
            }
        }
        component['listenerInstances'] = generateListnerInstances(ctx, component.listeners || [], {
            isComposite,
            looseError: options === null || options === void 0 ? void 0 : options.looseError,
        });
        delete component.listeners;
        if (!(0, common_1.isEmptyObj)(attributes)) {
            // 组件双向绑定
            const syncProps = componentInputProps[sourceKey];
            if (syncProps) {
                Object.keys(syncProps).forEach((syncPropKey) => {
                    let syncPropArr /*ISyncProp[]*/ = [];
                    const syncProp = syncProps[syncPropKey];
                    // 统一转成数组处理
                    if (!Array.isArray(syncProp)) {
                        syncPropArr = [syncProp];
                    }
                    else {
                        syncPropArr = syncProp;
                    }
                    syncPropArr.forEach(({ changeEvent, valueFromEvent }) => {
                        // 双向绑定需要优先第一个执行，否则会出现输入框的拼音被打断的问题
                        function withContext(context) {
                            const funcCode = `({ event, forItems }) => {
                const $for = forItems;
                const wid = ${isComposite ? 'this.widgets' : '$page.widgets'}.${component.id};
                // const widgetData = $for.forIndexes && $for.forIndexes.length > 0 ? context.get(wid, forItems.forIndexes) : wid;
                const widgetData = $for.forIndexes && $for.forIndexes.length > 0 ? get(wid, forItems.forIndexes) : wid;
                widgetData.${syncPropKey} = ${valueFromEvent}
              }`;
                            // let instance = new Function('context', `return ${funcCode}`)(context);
                            // return ctx.this ? instance.bind(ctx.this) : instance;
                            return `${config_1.REPLACE_SIGN}${funcCode}${config_1.REPLACE_SIGN}`;
                        }
                        component['listenerInstances'].unshift({
                            trigger: changeEvent,
                            instanceFunction: withContext({ get: lodash_1.get }),
                        });
                    });
                });
            }
        }
    });
    return {
        widgets: (0, util_1.JsonToStringWithVariableName)(compWidgets),
        dataBinds: (0, util_1.JsonToStringWithVariableName)(compDataBinds),
        componentSchema: (0, util_1.JsonToStringWithVariableName)(componentSchemaJson),
    };
}
exports.getComponentSchema = getComponentSchema;
// convert data binds to functions for performance & simplicity
function generateDataBinds(ctx, dataBinds, options) {
    const dataBindFuncs = {};
    dataBinds.forEach((bind) => {
        if (!bind.bindDataPath) {
            return console.warn('无 bindDataPath', bind.propertyPath);
        }
        let databindFunction = `() => {}`;
        try {
            let funcCode = `function (){}`;
            if (bind.type === cals_1.PropBindType.forItem) {
                funcCode = `(forItems) => forItems.${bind.bindDataPath}`;
            }
            else if (bind.type === 'scope') {
                funcCode = `() => ({__type: "scopedValue", getValue: ($scope)=>$scope.${bind.bindDataPath}})}`;
            }
            else if (bind.type === cals_1.PropBindType.expression) {
                let imports = 'const $for = forItems;';
                const expression = bind.bindDataPath.replace(/;$/, '');
                try {
                    imports += (0, util_1.generateExpressionAlias)(expression);
                }
                catch (e) {
                    console.error('parse expression error', expression, e === null || e === void 0 ? void 0 : e.message);
                }
                const body = `${imports} return (\n${expression || 'undefined'}\n)`;
                if (options === null || options === void 0 ? void 0 : options.isComposite) {
                    funcCode = `(forItems, $context) => {const $comp = this.$WEAPPS_COMP; ${body}}`;
                }
                else {
                    funcCode =
                        /(?<!\w)\$scope\./.test(expression) || expression === '$scope'
                            ? `(forItems, event, $context) => ({__type: "scopedValue", getValue: ($scope) => {${body}}})`
                            : `(forItems, event, $context) => {${body}}`;
                }
            }
            else if (bind.type === cals_1.PropBindType.prop) {
                let bindDataPath = bind.bindDataPath;
                const isNegated = bindDataPath.startsWith('!');
                if (isNegated)
                    bindDataPath = bindDataPath.replace(/^!/, '');
                if (options === null || options === void 0 ? void 0 : options.isComposite) {
                    // 复合组件在预览时其实就是page，所以沿用page的变量即可
                    funcCode = `() => ${isNegated ? '!' : ''}${'this.$WEAPPS_COMP'}.props.data.${bindDataPath}`;
                }
                else {
                    // 复合组件在预览时其实就是page，所以沿用page的变量即可
                    funcCode = `() => ${isNegated ? '!' : ''}$page.props.data.${bindDataPath}`;
                }
                throw new Error(funcCode);
            }
            else {
                const isGlobal = bind.bindDataPath.startsWith('global.');
                let returnValue = '';
                const PREFIX_MAP = {
                    [cals_1.PropBindType.state]: 'state',
                    [cals_1.PropBindType.computed]: 'computed',
                    // [PropBindType.dataVar]: 'dataVar',
                    [cals_1.PropBindType.stateData]: 'dataset.state',
                    [cals_1.PropBindType.paramData]: 'dataset.params',
                };
                switch (bind.type) {
                    case cals_1.PropBindType.state:
                    case cals_1.PropBindType.computed:
                    // case PropBindType.dataVar:
                    case cals_1.PropBindType.stateData:
                    case cals_1.PropBindType.paramData: {
                        const alias = PREFIX_MAP[bind.type];
                        if (isGlobal) {
                            returnValue = bind.bindDataPath.replace(/^global\./, `app.${alias}.`);
                        }
                        else {
                            if (options === null || options === void 0 ? void 0 : options.isComposite) {
                                returnValue = bind.bindDataPath
                                    .replace(/^comp-\w+([.[])/, (_, $1) => `this.$WEAPPS_COMP.${alias}${$1}`)
                                    .replace(/^\$\w+_\d+([.[])/, (_, $1) => `this.$WEAPPS_COMP.${alias}${$1}`);
                            }
                            else {
                                returnValue = bind.bindDataPath.replace(/^\w+([.[])/, (_, $1) => `$page.${alias}${$1}`);
                            }
                        }
                        funcCode = `() => ${returnValue}`;
                        break;
                    }
                }
            }
            // databindFunction = new Function(
            //   'context',
            //   `const { $page, app, $app } = context || {};
            //   return () => ${funcCode}`,
            // )(ctx);
            databindFunction = `${config_1.REPLACE_SIGN}${funcCode}${config_1.REPLACE_SIGN}`;
        }
        catch (e) {
            if (options === null || options === void 0 ? void 0 : options.looseError) {
                console.error('generate dynamic data error:', e);
                databindFunction = `${config_1.REPLACE_SIGN}() => {
          throw e;
        }${config_1.REPLACE_SIGN}`;
            }
            else {
                throw e;
            }
        }
        dataBindFuncs[bind.propertyPath] = databindFunction;
    });
    return dataBindFuncs;
}
function generateListnerInstances(ctx, listenerInstances, options = {
    isComposite: false,
}) {
    const { isComposite = false } = options;
    return listenerInstances.map((listener) => {
        var _a, _b, _c, _d, _e, _f;
        const generatedListener = {
            key: listener.id,
            trigger: listener.eventName,
            isCapturePhase: !!listener.isCapturePhase,
            noPropagation: listener['noPropagation'],
            instanceFunction: `${config_1.REPLACE_SIGN}() => {}${config_1.REPLACE_SIGN}`,
            data: {},
            dataBinds: {},
        };
        let funcCode = '()=>{}';
        switch (listener.type) {
            case cals_1.ActionType.Material: {
                const { name, module = cals_1.OFFICIAL_LIB_KEY } = listener.handler || {};
                funcCode = `(...args) => actionsMap["${module}:${name}"]?.apply?.(null, args)`;
                break;
            }
            case cals_1.ActionType.PropEvent: {
                if (isComposite) {
                    funcCode = `function({ data, event }){
            event.detail.data = data;
            this.props.emit('${((_a = listener === null || listener === void 0 ? void 0 : listener.handler) === null || _a === void 0 ? void 0 : _a.name) || listener['jsCode']}', event.detail)}.bind(this)`; // todo: dynamic bind this;
                }
                else {
                    funcCode = `function(...args) {
            return context.$page.props.events.${((_b = listener.handler) === null || _b === void 0 ? void 0 : _b.name) || listener['jsCode']}.apply(null, args)
          }`;
                }
                break;
            }
            case cals_1.ActionType.Inline: {
                // 内联 handler
                funcCode = `function({event, data, forItems, $w, args}){return ${listener['jsCode']}}`;
                if (isComposite) {
                    funcCode += '.bind(this)';
                }
                else {
                    funcCode += '.bind($page)';
                    // generatedListener.instanceFunction = instance.bind(ctx.$page);
                }
                break;
            }
            case cals_1.ActionType.GeneralFunc: {
                funcCode = `function({event, $w, params}) {
          return (\n${((_c = listener.handler) === null || _c === void 0 ? void 0 : _c.code) || '()=>{}'}\n)({event})
        }`;
                break;
            }
            case cals_1.ActionType.Platform: {
                let name = ((_d = listener.handler) === null || _d === void 0 ? void 0 : _d.name) || listener['jsCode'];
                funcCode = `({data})=> app.${name}(data)`;
                break;
            }
            case cals_1.ActionType.DataSource: {
                funcCode = `({data})=>{return app.cloud.callDataSource(data)}`;
                break;
            }
            default: {
                if (isComposite) {
                    funcCode = `this.handler.${((_e = listener.handler) === null || _e === void 0 ? void 0 : _e.name) || listener['jsCode']}.bind(this)`;
                }
                else {
                    let name = ((_f = listener.handler) === null || _f === void 0 ? void 0 : _f.name) || listener['jsCode'];
                    funcCode = `$page.handler.${name}`;
                }
            }
        }
        generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}${funcCode}${config_1.REPLACE_SIGN}`;
        const processedParams = (0, cals_1.deserializeDynamicMap)({
            page: { id: ctx.$page.id, type: cals_1.EComponentType.PAGE },
        }, listener.handler ? (0, cals_1.generateHandlerParamsDynamicMap)(listener.handler) : undefined);
        const { staticData, dataBinds } = splitDynamicMap(processedParams);
        generatedListener.data = staticData;
        generatedListener.dataBinds = generateDataBinds(ctx, dataBinds, options);
        return generatedListener;
    });
}
