"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateCompLibs = exports.generateCode = exports.TEMPLATE_REGEX = exports.deserialize = exports.serialize = void 0;
const generate_1 = require("./generate");
const material_1 = require("./material");
const common_1 = require("../types/common");
const template_1 = __importDefault(require("../template"));
const path_1 = __importDefault(require("path"));
const types_1 = require("../../types");
const cals_1 = require("@cloudbase/cals");
const config_1 = require("../config");
var weapps_core_1 = require("../../weapps-core");
Object.defineProperty(exports, "serialize", { enumerable: true, get: function () { return weapps_core_1.serialize; } });
Object.defineProperty(exports, "deserialize", { enumerable: true, get: function () { return weapps_core_1.deserialize; } });
exports.TEMPLATE_REGEX = /\.tpl(\.|$)/;
async function generateCode(props) {
    const { cals, mainAppData, subAppDataList = [], dependencies, appKey = 'test', i18nConfig = {
        enabled: false,
        i18nData: {},
    }, extraData = {
        isComposite: false,
        compProps: {},
    }, basename = '/', isSandbox = false, buildTypeList = [common_1.BuildType.WEB], ignoreInstall = false, deployOptions = { mode: types_1.DEPLOY_MODE.PREVIEW }, runtime = types_1.RUNTIME.NONE, _indexPage = false, isPrivateMode = false, expirationStartTimesnap, } = props;
    // 处理应用数据
    const fileCodeMap = {};
    const appBuildDir = '/';
    const includeRegex = /^\/src\/(utils|app)/;
    const FEATURE_MAP = (0, cals_1.parseVersion)(cals.schemaVersion, dependencies);
    const processCssUnit = FEATURE_MAP.defaultDynamicCssUnit ? 'rem' : 'px';
    const rpxCalcRoot = `var(${config_1.CALC_RPX_ROOT_VAL_NAME})`;
    copyFiles({
        appBuildDir,
        excludeRegex: exports.TEMPLATE_REGEX,
        includeRegex,
        fileCodeMap,
    });
    // i8n数据
    // fileCodeMap[path.join(appBuildDir, 'src/i18n/index.js')] = {
    //   code: `export default ${JSON.stringify(i18nConfig.i18nData, null, 2)}`,
    // };
    /**
     * 处理素材库
     * 符合组件 px 先转 rpx，再转 calc
     * 应用内处理规则须与单独打包组件规则一致
     */
    if (!isSandbox) {
        await (0, material_1.runHandleMaterial)(appBuildDir, dependencies, i18nConfig, fileCodeMap, isSandbox, 'rpx', rpxCalcRoot);
    }
    // 生成核心文件
    await (0, generate_1.runGenerateCore)({
        appBuildDir,
        appData: mainAppData,
        cals,
        subAppDataList,
        dependencies,
        appKey,
        buildTypeList,
        basename,
        extraData,
        i18nConfig,
        fileCodeMap,
        isSandbox,
        ignoreInstall,
        deployOptions,
        runtime,
        _indexPage,
        processCssUnit,
        rpxCalcRoot,
        isPrivateMode,
        expirationStartTimesnap,
    });
    return fileCodeMap;
}
exports.generateCode = generateCode;
async function generateCompLibs(compLibs, rpxCalcRoot = `var(${config_1.CALC_RPX_ROOT_VAL_NAME})`) {
    const fileCodeMap = {};
    const dependencies = compLibs;
    const isSandbox = true;
    const i18nConfig = {};
    const appBuildDir = '/';
    /**
     * 处理素材库
     * 符合组件 px 先转 rpx，再转 calc
     * 应用内处理规则须与单独打包组件规则一致
     */
    await (0, material_1.runHandleMaterial)(appBuildDir, dependencies, i18nConfig, fileCodeMap, isSandbox, 'rpx', rpxCalcRoot);
    const includeRegex = /^\/src\/(utils|handlers)/;
    copyFiles({
        appBuildDir,
        excludeRegex: exports.TEMPLATE_REGEX,
        includeRegex,
        fileCodeMap,
    });
    return fileCodeMap;
}
exports.generateCompLibs = generateCompLibs;
function copyFiles({ appBuildDir, excludeRegex, includeRegex, fileCodeMap }) {
    Object.keys(template_1.default).map((filePath) => {
        if (includeRegex) {
            const isInclude = includeRegex.test(filePath);
            if (isInclude) {
                const isExclude = excludeRegex === null || excludeRegex === void 0 ? void 0 : excludeRegex.test(filePath);
                if (isExclude) {
                    return;
                }
                const dstFile = path_1.default.join(appBuildDir, filePath);
                fileCodeMap[dstFile] = {
                    code: template_1.default[filePath].code,
                };
            }
        }
    });
}
