import { CSSProperties, ICommonStyle } from './style';
import { ActionType } from './action';
import { IWeAppCode } from './lowcode';
import { IMiniprogramPlugin, IPlugin } from '../types/plugins';
import { HISTORY_TYPE } from '../../types';
import { IDataSourceQuery } from '@cloudbase/cals';
export interface IQueryData extends Omit<IDataSourceQuery, 'datasourceName' | 'method' | 'listeners'> {
    handler: {
        moduleName: string;
        name: string;
    };
    /**
     * @deprecated
     */
    data: {
        [prop: string]: IDynamicValue;
    };
    /**
     * { params: {
     *    value: []
     * },
     * params.0.xxx: {
     *    value: "xx"
     * }}
     */
    args: {
        [prop: string]: IDynamicValue;
    };
    listeners: IEventListener[];
    /**
     * @deprecated
     */
    method: Omit<IDataSourceQuery['method'], 'params'> & {
        params: {
            [prop: string]: IDynamicValue;
        };
    };
    /**
     * @deprecated
     */
    datasourceName: string;
}
export interface IDatasetData {
    /** 各种变量 */
    dataset?: {
        state?: {};
        params?: {};
        query?: Record<string, IQueryData>;
    };
    vars: {
        /** 数据源变量 */
        data: any[];
    };
}
export interface IWeAppData {
    label?: string;
    description?: string;
    /**
     * 扩展支持非描述类型的应用包
     * @type {string}
     * @memberof IWeAppData
     */
    mpPkgUrl?: string;
    selectedPageId?: string;
    historyType?: HISTORY_TYPE;
    pageInstanceList: IWeAppPage[];
    lowCodes: IWeAppCode[];
    npmDependencies: {
        [packageName: string]: string;
    };
    plugins: IPlugin[];
    maxID?: number;
    rootPath?: string;
    themeVars?: {
        [key: string]: string;
    };
    presetColors?: string[];
    appConfig?: Record<string, any>;
    envId: string;
    datasources: any[];
    dataset?: IDatasetData['dataset'];
    vars: IDatasetData['vars'];
    miniprogramPlugins: IMiniprogramPlugin[];
    extra: {
        domain: string;
        clientId?: string;
        conditionalRendering?: boolean;
        enableLoading?: boolean;
        favicon?: string;
        enableLoginStatusShare?: boolean;
        /**
         *  自定义状态页id
         */
        statusPageId?: string;
        /**
         * 是否屏蔽访问
         */
        disabled?: string;
    };
}
export interface IWeAppPage {
    id: string;
    isHome: boolean;
    data: {
        [prop: string]: IDynamicValue;
    };
    listeners: IEventListener[];
    componentInstances: {
        [key: string]: IWeAppComponentInstance;
    };
    pluginInstances: any[];
    commonStyle: ICommonStyle;
    styleBindPath: string;
    lowCodes?: IWeAppCode[];
    children?: IWeAppPage[];
    pageId?: string;
    /**
     * @deprecated  moved to lowCodes
     */
    codeType?: 'page' | 'global';
    /**
     * @deprecated moved to lowCodes
     */
    code?: string;
    dataset?: IDatasetData['dataset'];
    eventFlows?: {
        id: string;
        label?: string;
        description?: string;
        listeners: IEventListener[];
    }[];
}
export interface IWeAppComponentInstance {
    xComponent?: {
        moduleName: string;
        name: string;
    };
    xProps?: {
        data: {
            [prop: string]: IDynamicValue;
        };
        directives?: {
            waIf?: IDynamicValue;
            waDisplay?: IDynamicValue;
            waFor?: IDynamicValue;
            waForKey?: IDynamicValue;
        };
        listeners?: IEventListener[];
        style: CSSProperties;
        styleBind: IDynamicValue;
        classList: string[];
        classListBind: IDynamicValue;
        commonStyle: ICommonStyle;
        scopedStyle?: string;
        styleBindPath?: string;
        customDataForm: Record<string, any>;
        staticResourceAttribute: string[];
    };
    xIndex?: number;
    properties?: {
        [key: string]: IWeAppComponentInstance;
    };
    /**
     * 用于组件开发，将当前组件设为抽象组件（用户在使用时可指定其他组件代替改组件）
     * https://developers.weixin.qq.com/miniprogram/dev/framework/custom-component/generics.html
     */
    genericComp?: {
        propName: string;
        title: string;
        description?: string;
        allowedCompType?: string;
        xIndex?: number;
    };
}
export interface IEventListener extends IEventModifiers {
    key?: string;
    trigger: string;
    type: ActionType;
    handler?: {
        moduleName: string;
        name: string;
        code?: string;
    };
    jsCode?: string;
    data: {
        [prop: string]: IDynamicValue;
    };
    args?: {
        /**
         * parasm: xxx
         * params.0.a: xxx
         */
        [prop: string]: IDynamicValue;
    };
}
export interface IEventModifiers {
    isCapturePhase?: boolean;
    noPropagation?: boolean;
}
export interface IDynamicValue {
    /**
     * computed: values derived from rematch store state by functions
     * rematch: data in rematch store state
     * local: local variables in the render function, e.g. for loop item
     */
    type?: 'static' | PropBindType;
    value: any | string;
}
export declare enum PropBindType {
    state = "rematch",
    computed = "computed",
    forItem = "for-item",
    expression = "expression",
    prop = "prop",
    slot = "slot",
    dataVar = "dataVar",
    stateData = "state",
    paramData = "params",
    scope = "scope"
}
export declare enum BindPropertyPath {
    style = "style",
    classNameList = "classNameList"
}
export interface ILockedPage {
    id: string;
    username: string;
    socketId: string;
}
export declare type IBuildType = 'web' | 'mp' | 'app' | 'qywxH5' | 'wxH5';
