import sdk from '@tcwd/weapps-sdk/lib/app-h5-sdk';
import { createWebApp, _ACTIONS_KEY as ACTIONS_KEY, _ROUTER_KEY as ROUTER_KEY } from '@cloudbase/weda-client';
import { createDataset, EXTRA_API } from '../datasources/index'

export const subPackageName = '<%= subPackageName %>';
export const app = createGlboalApi();
mountAPIs(sdk);
function createGlboalApi() {
  const mpApp = createWebApp();
  const globalAPI = {
    id: '<%= appId %>',
    domain: '<%= domain %>',
    formActions: {},
    pages: {},
    state: {},
    computed: {},
    common: {},
    ...mpApp,
    relaunchHome: function () {
      const { pages = [] } = globalAPI.utils._getConfig();
      if (pages[0]) {
        globalAPI.reLaunch({
          pageId: pages[0].id,
        });
      }
    },
    utils: {
      ...mpApp.utils,
      _getConfig: function () {
        return <%= appConfig %>;
      },
    },
    // i18n,
  };

  const dataset = createDataset('$global');
  globalAPI.dataset = dataset;
  globalAPI.state.dataset = dataset;
  globalAPI.setState = (userSetState) => {
    Object.keys(userSetState).forEach((keyPath) => {
      globalAPI.utils.set(
        globalAPI.dataset.state,
        keyPath,
        userSetState[keyPath]
      );
    });
  };
  /**
   * 内部通用的设置状态变量值的方法
   *  varPath 结构为 $global.<变量名> 即全局变量
   *                $page.<变量名>  即当前页面变量
   *                <pageId>.<变量名> 指定页面 pageId 的变量 (应当避免修改非当前页面的变量值)
   */
  globalAPI._setStateVal = (config) => {
    EXTRA_API.setState(config.varPath, config.val);
  };
  // 给全局挂上 mainApp/subApp
  // The global api exposed to lowcode
  if (subPackageName) {
    globalAPI.mainApp = window[`$$app`];
    window[`$$subapp_${subPackageName}`] = globalAPI;
  } else {
    window['app'] = globalAPI;
    window[`$$app`] = globalAPI;
  }

  // 挂运营平台上报对象到app里
  globalAPI.yyptReport = window.yyptReport;

  // 预览区调试栏可调试
  // 暴露给复合组件，代码组件来引用
  window.app = window.app || globalAPI;

  return globalAPI;
}

export function createPageApi() {
  const $page = new Proxy(
    {
      state: {},
      computed: {},
      handler: {},
      props: {},
      widgets: {},
      // 页面数据源变量存储位置
      dataVar: {},
    },
    {
      get(obj, prop) {
        return prop === '$page' ? obj : obj[prop];
      },
    }
  );
  return $page;
}

// 分app 和 wx 挂载app
function mountAPIs(sdks) {
  Object.keys(sdks).forEach((item) => {
    let action = sdks[item];
    if (!(item in ACTIONS_KEY) && !(item in ROUTER_KEY)) {
      app[item] = action;
    }
  });
  return app;
}
